/*
 * ImageAdjustor
 *
 *      A.Taniguchi(akira@behappy.2-d.jp)
 */

/*
 * ImageAdjustorBox object
 */

var ImageAdjustorBox = function(name, size, overview)
{
	this.IsGecko = (navigator.userAgent.toLowerCase().indexOf('gecko/') != -1);
	
    if (document.all)
    {
        if (document.getElementById && (document.compatMode == 'CSS1Compat')) {
		    this.IE_width = 0;
        } else {
		    this.IE_width = 2;
        }
    }
    else
    {
		    this.IE_width = 0;
	}
	
	this.name = name;
	this.size = size;
	this.overview = (overview) ? true : false;
	
	this.area = document.getElementById(this.name);
	
    this.area.style.position = 'relative';
    this.area.style.width = (this.size + this.IE_width) + 'px';
    this.area.style.height = (this.size + this.IE_width) + 'px';
    if (! overview) this.area.style.cursor = 'move';
    
	if (this.IsGecko)
	{
	    if (! overview) this.area.style.cursor = 'move';
	    
	    this.canvas = document.createElement('canvas');
	    this.canvas.width = this.size;
	    this.canvas.height = this.size;
	}
	else
	{
	    this.canvas = document.createElement('div');
	    this.canvas.style.width = (this.size + this.IE_width) + 'px';
	    this.canvas.style.height = (this.size + this.IE_width) + 'px';
	}
	
    this.canvas.style.margin = '0px';
    this.canvas.style.border = '1px solid red';
    this.canvas.style.padding = '0px';
    this.canvas.style.overflow = 'hidden';
    this.canvas.style.backgroundColor = 'white';
    this.canvas.style.position = 'absolute';
    this.canvas.style.top = 0;
    this.canvas.style.left = 0;
	
	if (this.IsGecko)
	{
	    this.image = new Image();
	    
	    this.framebox = document.createElement('div');
	    this.framebox.style.width = this.size + 'px';
		this.framebox.style.height = this.size + 'px';
	}
	else
	{
	    this.image = document.createElement('img');
	    this.image.style.margin = '0px';
	    this.image.style.borderWidth = '0px';
	    this.image.style.padding = '0px';
	    this.image.style.position = 'absolute';
	    this.image.style.top = 0;
	    this.image.style.left = 0;
	    this.image.style.filter = 'progid:DXImageTransform.Microsoft.Matrix(sizingMethod=\'auto expand\')';
	    
	    this.canvas.appendChild( this.image );
	    
	    this.framebox = document.createElement('div');
	    this.framebox.style.width = (this.size + this.IE_width) + 'px';
		this.framebox.style.height = (this.size + this.IE_width) + 'px';
	}
    
    this.framebox.style.margin = '0px';
    this.framebox.style.border = '1px solid blue';
    this.framebox.style.padding = '0px';
    this.framebox.style.position = 'absolute';
    this.framebox.style.top = 0;
    this.framebox.style.left = 0;
    this.framebox.style.zIndex = 12;
    this.framebox.style.backgroundColor = 'transparent';
    
    this.area.appendChild(this.canvas);
    this.area.appendChild(this.framebox);
}


ImageAdjustorBox.prototype.setImage = function(imagePath)
{
	this.image.src = imagePath;
}


ImageAdjustorBox.prototype.draw = function(width, height, rotate, scale, x, y)
{
	var dtr = Math.PI / 180;
	
	var adjustR = ((360 + rotate) % 360) * dtr;
    var adjustS = (scale < (1 / this.size)) ? (1 / this.size) : ((scale > this.size) ? this.size : scale);
    var adjustX = 0;
    var adjustY = 0;
    
    var imageWidth = Math.round(Math.abs(Math.cos(adjustR) * this.image.width - Math.sin(adjustR) * this.image.height));
    var imageHeight = Math.round(Math.abs(Math.cos(adjustR) * this.image.height + Math.sin(adjustR) * this.image.width));
    
	if (this.overview)
    {
    	var x1 = Math.min( -(imageWidth / 2 * adjustS) + x, -(width / 2) );
    	var x2 = Math.max( (imageWidth / 2 * adjustS) + x, width / 2 );
    	adjustWidth = (x2 - x1);
    	
    	var y1 = Math.min( -(imageHeight / 2 * adjustS) + y, -(height / 2) );
    	var y2 = Math.max( (imageHeight / 2 * adjustS) + y, height / 2 );
    	adjustHeight = (y2 - y1);
    	
	    var adjust = (Math.max(adjustWidth, adjustHeight) == 0) ? this.size : this.size / Math.max(adjustWidth, adjustHeight);
	    
	    var dx = this.size / 2 / adjust - (imageWidth / 2 * adjustS - (x + adjustX));
	    
	    if (dx < 0)
	    {
	    	adjustX -= dx;
	    }
	    
	    var dx = this.size / 2 / adjust - (imageWidth / 2 * adjustS + (x + adjustX));
	    
	    if (dx < 0)
	    {
	    	adjustX += dx;
	    }
	    
	    var dy = this.size / 2 / adjust - (imageHeight / 2 * adjustS - (y + adjustY));
	    
	    if (dy < 0)
	    {
	    	adjustY -= dy;
	    }
	    
	    var dy = this.size / 2 / adjust - (imageHeight / 2 * adjustS + (y + adjustY));
	    
	    if (dy < 0)
	    {
	    	adjustY += dy;
	    }
    }
    else
    {
	    var adjust = (Math.max(width, height) == 0) ? this.size : this.size / Math.max(width, height);
    }
	
	// framebox
	
    this.framebox.style.width = Math.round(width * adjust) + this.IE_width + 'px';
    this.framebox.style.height = Math.round(height * adjust) + this.IE_width + 'px';
	
    this.framebox.style.left = Math.round(this.size / 2 - (width / 2 - adjustX) * adjust) + 'px';
    this.framebox.style.top = Math.round(this.size / 2 - (height / 2 - adjustY) * adjust) + 'px';
    
	if (this.IsGecko)
	{
		// Because coordinates are scaled by Context,
		// the unit of coordinates is a pixel of the image. 
		
		var dx = (this.size / adjust / adjustS - this.image.width) / 2;
		var dy = (this.size / adjust / adjustS - this.image.height) / 2;
		
	    var ctx = this.canvas.getContext("2d");
	    
	    ctx.save();
	    ctx.clearRect(0, 0, this.size, this.size);
	    ctx.translate(this.size / 2 + (x + adjustX) * adjust, this.size / 2 + (y + adjustY) * adjust);
	    ctx.rotate(adjustR);
	    ctx.translate(-this.size / 2, -this.size / 2);
	    ctx.scale(adjustS * adjust, adjustS * adjust);
	    ctx.drawImage(this.image, dx, dy);
	    ctx.restore();
	}
	else
	{
	    // sizingMethod='auto expand' 座標補正
	    
		var sx = this.image.width / 2;
		var sy = this.image.height / 2;
		
		var x1 = Math.cos(adjustR) * -sx - Math.sin(adjustR) * -sy;
		var y1 = Math.cos(adjustR) * -sy + Math.sin(adjustR) * -sx;
		
		var x2 = Math.cos(adjustR) * sx - Math.sin(adjustR) * -sy;
		var y2 = Math.cos(adjustR) * -sy + Math.sin(adjustR) * sx;
		
		var x3 = Math.cos(adjustR) * sx - Math.sin(adjustR) * sy;
		var y3 = Math.cos(adjustR) * sy + Math.sin(adjustR) * sx;
		
		var x4 = Math.cos(adjustR) * -sx - Math.sin(adjustR) * sy;
		var y4 = Math.cos(adjustR) * sy + Math.sin(adjustR) * -sx;
		
	    var fx = -sx - Math.min(Math.min(x1, x2), Math.min(x3, x4));
	    var fy = -sy - Math.min(Math.min(y1, y2), Math.min(y3, y4));
		
	    // 座標設定
	    
	    this.image.style.left = Math.round(this.size / 2 - ((this.image.width / 2 + fx) * adjustS - (x + adjustX)) * adjust) + 'px';
	    this.image.style.top = Math.round(this.size / 2 - ((this.image.height / 2 + fy) * adjustS - (y + adjustY)) * adjust) + 'px';
	    
	    // 描画
	    
	    this.image.filters.item(0).M11 = Math.cos(adjustR) * adjustS * adjust;
	    this.image.filters.item(0).M12 = -Math.sin(adjustR) * adjustS * adjust;
	    this.image.filters.item(0).M21 = Math.sin(adjustR) * adjustS * adjust;
	    this.image.filters.item(0).M22 = Math.cos(adjustR) * adjustS * adjust;
	}
}


/*
 * ImageAdjustor object
 *
 * properties:
 *     IsGecko
 *
 *     rotate
 *     scale
 *     x
 *     y
 *
 *     width
 *     height
 *
 *     __width
 *     __height
 *
 *     preview
 *     overview
 */

var ImageAdjustor = function(previewName, previewSize, overviewName, overviewSize)
{
	this.IsGecko = (navigator.userAgent.toLowerCase().indexOf('gecko/') != -1);
	
	this.scale = 1.0;
	this.rotate = 0.0;
	this.x = 0.0;
	this.y = 0.0;
	
	this.preview = new ImageAdjustorBox(previewName, previewSize);
	
	if (overviewName)
	{
		this.overview = new ImageAdjustorBox(overviewName, overviewSize, true);
	}
}


ImageAdjustor.prototype.setImage = function(imagePath, width, height)
{
	this.width = width;
	this.height = height;
	
	// for reset
	
	this.__width = width;
	this.__height = height;
	
	this.preview.setImage(imagePath);
	if (this.overview) this.overview.setImage(imagePath);
}


ImageAdjustor.prototype.start = function()
{
    if(! this.preview.image.complete) return false;
    
    if(this.overview)
	    if(! this.overview.image.complete) return false;
    
	this.enableDraw = true;
	
    this.draw();
    
    return true;
}


ImageAdjustor.prototype.draw = function()
{
	if (! this.enableDraw) return;
	
	this.preview.draw(this.width, this.height, this.rotate, this.scale, this.x, this.y);
	
	if (this.overview)
	{
		this.overview.draw(this.width, this.height, this.rotate, this.scale, this.x, this.y);
	}
}


ImageAdjustor.prototype.reset = function()
{
	this.scale = 1.0;
	this.rotate = 0.0;
	this.x = 0.0;
	this.y = 0.0;
	
	this.width = this.__width;
	this.height = this.__height;
	
	this.draw();
}


ImageAdjustor.prototype.setPosition = function(x, y)
{
	this.x = x;
	this.y = y;
	this.draw();
}


ImageAdjustor.prototype.setScale = function(scale)
{
	this.scale = scale;
	this.draw();
}


ImageAdjustor.prototype.setRotate = function(degree)
{
	var dtr = Math.PI / 180;
	
    var x = (Math.cos(-this.rotate * dtr) * this.x) - (Math.sin(-this.rotate * dtr) * this.y);
    var y = (Math.cos(-this.rotate * dtr) * this.y) + (Math.sin(-this.rotate * dtr) * this.x);
    
    this.x = (Math.cos(degree * dtr) * x) - (Math.sin(degree * dtr) * y);
    this.y = (Math.cos(degree * dtr) * y) + (Math.sin(degree * dtr) * x);
    
	this.rotate = degree;
	
	this.draw();
}


ImageAdjustor.prototype.move = function(dx, dy)
{
	this.x += dx;
	this.y += dy;
	this.draw();
}


ImageAdjustor.prototype.zoom = function(scale)
{
	this.scale = this.scale * scale;
	this.draw();
}


ImageAdjustor.prototype.turn = function(degree)
{
	var dtr = Math.PI / 180;
	
    var x = Math.round((Math.cos(degree * dtr) * this.x) - (Math.sin(degree * dtr) * this.y));
    var y = Math.round((Math.cos(degree * dtr) * this.y) + (Math.sin(degree * dtr) * this.x));
    
    this.x = x;
    this.y = y;
    
	this.rotate = (360 + this.rotate + degree) % 360;
	
	this.draw();
}


ImageAdjustor.prototype.turnFrame = function()
{
    var w;
    var h;
    
    w = this.__width;
    h = this.__height;
    
    this.__width = h;
    this.__height = w;
	
    w = this.width;
    h = this.height;
    
    this.width = h;
    this.height = w;
	
	this.draw();
}


ImageAdjustor.prototype.fit = function(f)
{
	switch ( true )
	{
		case ( this.rotate % 180 == 0 ) :
			var imageWidth = this.preview.image.width ;
			var imageHeight = this.preview.image.height ;
			break ;
		
		case ( this.rotate % 90 == 0 ) :
			var imageWidth = this.preview.image.height ;
			var imageHeight = this.preview.image.width ;
			break ;
		
		default :
			var dtr = Math.PI / 180 ;
			var rotateCos = Math.cos( this.rotate * dtr ) ;
			var rotateSin = Math.sin( this.rotate * dtr ) ;
			
    		var imageWidth = Math.abs( rotateCos * this.preview.image.width - rotateSin * this.preview.image.height ) ;
    		var imageHeight = Math.abs( rotateCos * this.preview.image.height + rotateSin * this.preview.image.width ) ;
	}
    
    var scaleW = this.width / imageWidth ;
    var scaleH = this.height / imageHeight ;
    
    this.scale = Math.round( ( ( f ) ? Math.max( scaleW, scaleH ) : Math.min( scaleW, scaleH ) ) * 10000 ) / 10000 ;
    
    this.x = 0.0;
    this.y = 0.0;
	
	this.draw();
}


ImageAdjustor.prototype.connectorUrl = function( ImageAdjustorPath )
{
	var sUrl = ImageAdjustorPath ;
	sUrl += ImageAdjustorPath.match( /\/$/ ) ? '' : '/' ;
	sUrl += 'php/connector.php' ;
	sUrl += '?Source=' + encodeURIComponent( this.preview.image.src );
	sUrl += '&Rotate=' + this.rotate;
	sUrl += '&Scale=' + ( Math.round( this.scale * 10000 ) / 10000 ) ;
	sUrl += '&X=' + this.x;
	sUrl += '&Y=' + this.y;
	sUrl += '&Width=' + this.width;
	sUrl += '&Height=' + this.height;
	
	return sUrl;
}
